@@ -1,5 +1,4 @@
 # flask_restful_swagger2 API subclass
-import copy
 from http import HTTPStatus
 import logging
 import werkzeug
@@ -9,11 +8,10 @@ from flask_restful.utils import OrderedDict
 from flask_restful.utils import cors
 from flask_restful_swagger_2 import Api as FRSApiBase
 from flask_restful_swagger_2 import validate_definitions_object, parse_method_doc
-from flask_restful_swagger_2 import validate_path_item_object
+from flask_restful_swagger_2 import validate_path_item_object, Schema
 from flask_restful_swagger_2 import extract_swagger_path, Extractor, ValidationError as FRSValidationError
 from flask import request
 from functools import wraps
-import collections
 import safrs
 from .swagger_doc import swagger_doc, swagger_method_doc, default_paging_parameters
 from .swagger_doc import parse_object_doc, swagger_relationship_doc, get_http_methods
@@ -27,12 +25,12 @@ import json
 HTTP_METHODS = ["GET", "POST", "PATCH", "DELETE", "PUT"]
 DEFAULT_REPRESENTATIONS = [("application/vnd.api+json", output_json)]
 
-# pylint: disable=protected-access,invalid-name,line-too-long,logging-format-interpolation,fixme,too-many-branches
+
 class SAFRSAPI(FRSApiBase):
     """
-        Subclass of the flask_restful_swagger API class where we add the expose_object method
-        this method creates an API endpoint for the SAFRSBase object and corresponding swagger
-        documentation
+    Subclass of the flask_restful_swagger API class where we add the expose_object method
+    this method creates an API endpoint for the SAFRSBase object and corresponding swagger
+    documentation
     """
 
     _operation_ids = {}
@@ -49,16 +47,16 @@ class SAFRSAPI(FRSApiBase):
         **kwargs
     ):
         """
-            http://jsonapi.org/format/#content-negotiation-servers
-            Servers MUST send all JSON:API data in response documents with
-            the header Content-Type: application/vnd.api+json without any media type parameters.
+        http://jsonapi.org/format/#content-negotiation-servers
+        Servers MUST send all JSON:API data in response documents with
+        the header Content-Type: application/vnd.api+json without any media type parameters.
 
-            Servers MUST respond with a 415 Unsupported Media Type status code if
-            a request specifies the header Content-Type: application/vnd.api+json with any media type parameters.
+        Servers MUST respond with a 415 Unsupported Media Type status code if
+        a request specifies the header Content-Type: application/vnd.api+json with any media type parameters.
 
-            Servers MUST respond with a 406 Not Acceptable status code if
-            a request’s Accept header contains the JSON:API media type and
-            all instances of that media type are modified with media type parameters.
+        Servers MUST respond with a 406 Not Acceptable status code if
+        a request’s Accept header contains the JSON:API media type and
+        all instances of that media type are modified with media type parameters.
         """
 
         custom_swagger = kwargs.pop("custom_swagger", {})
@@ -86,49 +84,47 @@ class SAFRSAPI(FRSApiBase):
 
     def update_spec(self, custom_swagger):
         """
-            :param custom_swagger: swagger spec to be added to the swagger.json
+        :param custom_swagger: swagger spec to be added to the swagger.json
         """
         _swagger_doc = self.get_swagger_doc()
         safrs.dict_merge(_swagger_doc, custom_swagger)
 
     def expose_object(self, safrs_object, url_prefix="", **properties):
-        """ This methods creates the API url endpoints for the SAFRObjects
-            :param safrs_object: SAFSBase subclass that we would like to expose
-            :param url_prefix: url prefix
-            :param properties: additional flask-restful properties
+        """This methods creates the API url endpoints for the SAFRObjects
+        :param safrs_object: SAFSBase subclass that we would like to expose
+        :param url_prefix: url prefix
+        :param properties: additional flask-restful properties
 
-            creates a class of the form
+        creates a class of the form
 
-            @api_decorator
-            class Class_API(SAFRSRestAPI):
-                SAFRSObject = safrs_object
+        @api_decorator
+        class Class_API(SAFRSRestAPI):
+            SAFRSObject = safrs_object
 
-            add the class as an api resource to /SAFRSObject and /SAFRSObject/{id}
+        add the class as an api resource to /SAFRSObject and /SAFRSObject/{id}
 
-            tablename/collectionname: safrs_object._s_collection_name, e.g. "Users"
-            classname: safrs_object.__name__, e.g. "User"
+        tablename/collectionname: safrs_object._s_collection_name, e.g. "Users"
+        classname: safrs_object.__name__, e.g. "User"
         """
+        properties["SAFRSObject"] = safrs_object
+        properties["http_methods"] = safrs_object.http_methods
         safrs_object.url_prefix = url_prefix
-        api_class_name = "{}_API".format(safrs_object._s_type)
+        endpoint = safrs_object.get_endpoint()
 
         # tags indicate where in the swagger hierarchy the endpoint will be shown
         tags = [safrs_object._s_collection_name]
-        properties["SAFRSObject"] = safrs_object
-        properties["http_methods"] = safrs_object.http_methods
 
         # Expose the methods first
         self.expose_methods(url_prefix, tags, safrs_object, properties)
 
-        RESOURCE_URL_FMT = get_config("RESOURCE_URL_FMT")
-        url = RESOURCE_URL_FMT.format(url_prefix, safrs_object._s_collection_name)
-
-        endpoint = safrs_object.get_endpoint()
-
         # Expose the collection: Create the class and decorate it
+        api_class_name = "{}_API".format(safrs_object._s_type)  # name for dynamically generated classes
+        RESOURCE_URL_FMT = get_config("RESOURCE_URL_FMT")  # configurable resource collection url formatter
+        url = RESOURCE_URL_FMT.format(url_prefix, safrs_object._s_collection_name)
         swagger_decorator = swagger_doc(safrs_object)
         api_class = api_decorator(type(api_class_name, (SAFRSRestAPI,), properties), swagger_decorator)
 
-        safrs.log.info("Exposing {} on {}, endpoint: {}".format(safrs_object._s_type, url, endpoint))
+        safrs.log.info("Exposing {} on {}, endpoint: {}".format(safrs_object._s_collection_name, url, endpoint))
         self.add_resource(api_class, url, endpoint=endpoint, methods=["GET", "POST"])
 
         INSTANCE_URL_FMT = get_config("INSTANCE_URL_FMT")
@@ -136,21 +132,34 @@ class SAFRSAPI(FRSApiBase):
         endpoint = safrs_object.get_endpoint(type="instance")
 
         # Expose the instances
-        safrs.log.info("Exposing {} instances on {}, endpoint: {}".format(safrs_object._s_collection_name, url, endpoint))
-        self.add_resource(api_class, url, endpoint=endpoint)
+        safrs.log.info("Exposing {} instances on {}, endpoint: {}".format(safrs_object._s_type, url, endpoint))
+        api_class = api_decorator(type(api_class_name + "_i", (SAFRSRestAPI,), properties), swagger_decorator)
+        self.add_resource(api_class, url, endpoint=endpoint, methods=["GET", "PATCH", "DELETE"])
 
         object_doc = parse_object_doc(safrs_object)
         object_doc["name"] = safrs_object._s_collection_name
         self._swagger_object["tags"].append(object_doc)
 
-        for rel_name, relationship in safrs_object._s_relationships.items():
+        for relationship in safrs_object._s_relationships.values():
             self.expose_relationship(relationship, url, tags, properties)
 
+        # add newly created schema references to the "definitions"
+        for def_name, definition in Schema._references.items():
+            if self._swagger_object["definitions"].get(def_name):
+                continue
+            try:
+                validate_definitions_object(definition.properties)
+            except Exception as exc:  # pragma: no cover
+                safrs.log.warning("Failed to validate {}:{}".format(definition, exc))
+                continue
+            self._swagger_object["definitions"][def_name] = {"properties": definition.properties}
+
     def expose_methods(self, url_prefix, tags, safrs_object, properties):
-        """ Expose the safrs "documented_api_method" decorated methods
-            :param url_prefix: api url prefix
-            :param tags: swagger tags
-            :return: None
+        """
+        Expose the safrs "documented_api_method" decorated methods
+        :param url_prefix: api url prefix
+        :param tags: swagger tags
+        :return: None
         """
 
         api_methods = safrs_object._s_get_jsonapi_rpc_methods()
@@ -180,33 +189,32 @@ class SAFRSAPI(FRSApiBase):
 
     def expose_relationship(self, relationship, url_prefix, tags, properties):
         """
-            Expose a relationship tp the REST API:
-            A relationship consists of a parent and a target class
-            creates a class of the form
+        Expose a relationship tp the REST API:
+        A relationship consists of a parent and a target class
+        creates a class of the form
 
-            @api_decorator
-            class Parent_X_target_API(SAFRSRestAPI):
-                SAFRSObject = safrs_object
+        @api_decorator
+        class Parent_X_target_API(SAFRSRestAPI):
+            SAFRSObject = safrs_object
 
-            add the class as an api resource to /SAFRSObject and /SAFRSObject/{id}
+        add the class as an api resource to /SAFRSObject and /SAFRSObject/{id}
 
-            :param relationship: relationship
-            :param url_prefix: api url prefix
-            :param tags: swagger tags
-            :return: None
+        :param relationship: relationship
+        :param url_prefix: api url prefix
+        :param tags: swagger tags
+        :return: None
         """
         # safrs_object is the target class, if this is not a SAFRSBase class, then we shouldn't expose it
         # the _s_expose attribute indicates we're dealing with a SAFRSBase instance
         # if the relationship is not an sql sqlalchemy.orm.relationships.RelationshipProperty instance
         # then we should have defined the _target
         target_object = relationship.mapper.class_
-        if not getattr(target_object, "_s_expose", False):
+        if not getattr(target_object, "_s_expose", False):  # todo: add test
             safrs.log.debug("Not exposing {}".format(target_object))
             return
 
-        API_CLASSNAME_FMT = "{}_X_{}_API"
+        API_CLASSNAME_FMT = "{}_X_{}_API"  # api class name for generated relationship classes
         rel_name = relationship.key
-
         parent_class = relationship.parent.class_
         parent_name = parent_class.__name__
 
@@ -272,14 +280,15 @@ class SAFRSAPI(FRSApiBase):
 
         safrs.log.info("Exposing {} relationship {} on {}, endpoint: {}".format(parent_name, rel_name, url, endpoint))
         self.add_resource(
-            api_class, url, relationship=rel_object.relationship, endpoint=endpoint, methods=["GET", "DELETE"], deprecated=True
+            api_class, url, relationship=rel_object.relationship, endpoint=endpoint, methods=["GET", "PATCH", "DELETE"], deprecated=True
         )
 
     @staticmethod
     def get_resource_methods(resource, ordered_methods=None):
-        """ :param ordered_methods:
-            :return: the http methods from the SwaggerEndpoint and SAFRS Resources,
-            in the order specified by ordered_methods
+        """
+        :param ordered_methods:
+        :return: the http methods from the SwaggerEndpoint and SAFRS Resources,
+        in the order specified by ordered_methods
         """
         if ordered_methods is None:
             ordered_methods = HTTP_METHODS
@@ -293,30 +302,181 @@ class SAFRSAPI(FRSApiBase):
 
     def add_resource(self, resource, *urls, **kwargs):
         """
-            This method is partly copied from flask_restful_swagger_2/__init__.py
+        This method is partly copied from flask_restful_swagger_2/__init__.py
 
-            I changed it because we don't need path id examples when
-            there's no {id} in the path.
-            We also have to filter out the unwanted parameters
+        Changed because we don't need path id examples when there's no {id} in the path.
+        We also have to filter out the unwanted parameters
         """
-        #
-        # This function has grown out of proportion and should be refactored, disable lint warning for now
-        #
-        # pylint: disable=too-many-nested-blocks,too-many-statements, too-many-locals
-        #
-        kwargs.pop("relationship", False)  # relationship object
+        relationship = kwargs.pop("relationship", False)  # relationship object
         SAFRS_INSTANCE_SUFFIX = get_config("OBJECT_ID_SUFFIX") + "}"
 
-        path_item = collections.OrderedDict()
-        definitions = {}
-        resource_methods = kwargs.get("methods", HTTP_METHODS)
-        kwargs.pop("safrs_object", None)
+        path_item = {}
+        self._add_oas_resource_definitions(resource, path_item)
         is_jsonapi_rpc = kwargs.pop("jsonapi_rpc", False)  # check if the exposed method is a jsonapi_rpc method
-        deprecated = kwargs.pop("deprecated", False)  # TBD!!
-        for method in self.get_resource_methods(resource):
+        deprecated = kwargs.pop("deprecated", False)  # deprecated functionality: still working but not shown in swagger
+
+        # this loop builds the swagger for the specified url(s) by adding it to
+        # self._swagger_object["paths"][swagger_url], so if the loop continues,
+        # there will be no swagger
+        # usually there will only be one url, but flask_restful add_resource does support multiple urls
+        for url in urls:
             if deprecated:
+                # functionality still works, but there will be no swagger
                 continue
-            if not method.upper() in resource_methods:
+            if not url.startswith("/"):  # pragma: no cover
+                raise ValidationError("paths must start with a /")
+
+            swagger_url = extract_swagger_path(url)
+            # exposing_instance tells us whether we're exposing an instance (as opposed to a collection)
+            exposing_instance = swagger_url.strip("/").endswith(SAFRS_INSTANCE_SUFFIX)
+            for method in self.get_resource_methods(resource):
+                if kwargs.get("methods", None) and method.upper() not in [m.upper() for m in kwargs.get("methods", [])]:
+                    # only use the
+                    path_item.pop(method, None)
+                    continue
+
+                if method == "post" and exposing_instance:
+                    # POSTing to an instance isn't jsonapi-compliant (https://jsonapi.org/format/#crud-creating-client-ids)
+                    # "A server MUST return 403 Forbidden in response to an
+                    # unsupported request to create a resource with a client-generated ID"
+                    # the method has already been added before, remove it & continue
+                    path_item.pop(method, None)
+                    continue
+
+                method_doc = path_item.get(method)
+                if not method_doc:
+                    continue
+
+                # exposed objects and methods may specify `collection_summary` in the method docstring yaml
+                collection_summary = method_doc.pop("collection_summary", method_doc.get("summary", None))
+                if not exposing_instance and collection_summary:
+                    method_doc["summary"] = collection_summary
+
+                method_doc["operationId"] = self._get_operation_id(path_item.get(method).get("summary", ""))
+
+                self._add_oas_req_params(resource, path_item, method, exposing_instance, is_jsonapi_rpc, swagger_url)
+                self._add_oas_references(resource.SAFRSObject, path_item, method, exposing_instance, relationship)
+
+                try:  # pragma: no cover
+                    validate_path_item_object(path_item)
+                except FRSValidationError as exc:
+                    safrs.log.exception(exc)
+                    safrs.log.critical("Validation failed for {}".format(path_item))
+                    exit(1)
+
+            self._swagger_object["paths"][swagger_url] = path_item
+            # Check whether we manage to convert to json
+            try:
+                json.dumps(self._swagger_object)
+            except Exception:  # pragma: no cover
+                safrs.log.critical("Json encoding failed")
+
+        # disable API methods that were not set by the SAFRSObject
+        for http_method in HTTP_METHODS:
+            hm = http_method.lower()
+            if hm not in self.get_resource_methods(resource):
+                setattr(resource, hm, lambda x: ({}, HTTPStatus.METHOD_NOT_ALLOWED))
+
+        super(FRSApiBase, self).add_resource(resource, *urls, **kwargs)
+
+    def _add_oas_req_params(self, resource, path_item, method, exposing_instance, is_jsonapi_rpc, swagger_url):
+        """
+
+        Add the request parameters to the swagger (filter, sort)
+        """
+        method_doc = path_item[method]
+        parameters = []
+        for parameter in method_doc.get("parameters", []):
+            object_id = "{%s}" % parameter.get("name")
+            if method == "get":
+                # Get the jsonapi included resources, ie the exposed relationships
+                param = resource.get_swagger_include()
+                parameters.append(param)
+
+                # Get the jsonapi fields[], ie the exposed attributes/columns
+                param = resource.get_swagger_fields()
+                parameters.append(param)
+
+            #
+            # Add the sort, filter parameters to the swagger doc when retrieving a collection
+            #
+            if method == "get" and not (exposing_instance or is_jsonapi_rpc):
+                # limit parameter specifies the number of items to return
+                parameters += default_paging_parameters()
+                param = resource.get_swagger_sort()
+                parameters.append(param)
+                parameters += list(resource.get_swagger_filters())
+
+            if not (parameter.get("in") == "path" and object_id not in swagger_url) and parameter not in parameters:
+                # Only if a path param is in path url then we add the param
+                parameters.append(parameter)
+
+        unique_params = OrderedDict()  # rm duplicates
+        for param in parameters:
+            unique_params[param["name"]] = param
+        method_doc["parameters"] = list(unique_params.values())
+        path_item[method] = method_doc
+
+    def _add_oas_references(self, safrs_object, path_item, method, exposing_instance, relationship):
+        """
+        substitute the swagger references in the response objects
+        references are created and added to the safrs_object.swagger_models in swagger_doc
+
+        the params are
+        :param safrs_object:
+        :param path_item:
+
+        """
+        inst_ref = None
+        coll_ref = None
+
+        if getattr(safrs_object, "swagger_models", {}).get("instance"):
+            # instance reference
+            inst_ref = safrs_object.swagger_models["instance"].reference()
+
+        if getattr(safrs_object, "swagger_models", {}).get("collection"):
+            # collection reference
+            coll_ref = safrs_object.swagger_models["collection"].reference()
+
+        if not inst_ref and not coll_ref:
+            return
+
+        method_doc = path_item[method]
+        response = method_doc.get("responses", {})
+        if "200" in response:
+            # add the "example" response schema references
+            if exposing_instance:
+                response["200"]["schema"] = inst_ref
+            elif coll_ref:
+                response["200"]["schema"] = coll_ref
+            response["200"].pop("type", None)
+
+        if "201" in response:
+            if method == "post":
+                # Posting to a collection returns the instance (except when using bulk post, but this isn't shown atm)
+                response["201"]["schema"] = inst_ref
+            elif exposing_instance:
+                # patching an instance
+                response["201"]["schema"] = inst_ref
+            elif coll_ref:
+                # patching a
+                response["201"]["schema"] = coll_ref
+            response["201"]["schema"].pop("type", None)
+
+        if relationship:
+            print(method_doc.get("responses", {}).get("200", {}))
+            # print(relationship.direction)
+
+    def _add_oas_resource_definitions(self, resource, path_item):
+        """
+        add the resource method schema references to the swagger "definitions"
+        :param resource:
+        :param path_item:
+        """
+        definitions = {}
+
+        for method in self.get_resource_methods(resource):
+            if not method.upper() in HTTP_METHODS:
                 continue
             f = getattr(resource, method, None)
             if not f:
@@ -324,7 +484,6 @@ class SAFRSAPI(FRSApiBase):
 
             operation = getattr(f, "__swagger_operation_object", None)
             if operation:
-                # operation, definitions_ = self._extract_schemas(operation)
                 operation, definitions_ = Extractor.extract(operation)
                 path_item[method] = operation
                 definitions.update(definitions_)
@@ -340,101 +499,10 @@ class SAFRSAPI(FRSApiBase):
 
         self._swagger_object["definitions"].update(definitions)
 
-        if path_item:
-            for url in urls:
-                if not url.startswith("/"):
-                    raise ValidationError("paths must start with a /")
-                swagger_url = extract_swagger_path(url)
-
-                # exposing_instance tells us whether we're exposing an instance (as opposed to a collection)
-                exposing_instance = swagger_url.strip("/").endswith(SAFRS_INSTANCE_SUFFIX)
-
-                for method in self.get_resource_methods(resource):
-                    if method == "post" and exposing_instance:
-                        # POSTing to an instance isn't jsonapi-compliant (https://jsonapi.org/format/#crud-creating-client-ids)
-                        # "A server MUST return 403 Forbidden in response to an
-                        # unsupported request to create a resource with a client-generated ID"
-                        # the method has already been added before, remove it & continue
-                        path_item.pop(method, None)
-                        continue
-
-                    method_doc = copy.deepcopy(path_item.get(method))
-                    if not method_doc:
-                        continue
-
-                    collection_summary = method_doc.pop("collection_summary", method_doc.get("summary", None))
-                    if not exposing_instance and collection_summary:
-                        method_doc["summary"] = collection_summary
-
-                    parameters = []
-                    for parameter in method_doc.get("parameters", []):
-                        object_id = "{%s}" % parameter.get("name")
-                        if method == "get":
-                            # Get the jsonapi included resources, ie the exposed relationships
-                            param = resource.get_swagger_include()
-                            parameters.append(param)
-
-                            # Get the jsonapi fields[], ie the exposed attributes/columns
-                            param = resource.get_swagger_fields()
-                            parameters.append(param)
-
-                        #
-                        # Add the sort, filter parameters to the swagger doc when retrieving a collection
-                        #
-                        if method == "get" and not (exposing_instance or is_jsonapi_rpc):
-                            # limit parameter specifies the number of items to return
-                            parameters += default_paging_parameters()
-
-                            param = resource.get_swagger_sort()
-                            parameters.append(param)
-
-                            parameters += list(resource.get_swagger_filters())
-
-                        if not (parameter.get("in") == "path" and object_id not in swagger_url) and parameter not in parameters:
-                            # Only if a path param is in path url then we add the param
-                            parameters.append(parameter)
-
-                    unique_params = OrderedDict()  # rm duplicates
-                    for param in parameters:
-                        unique_params[param["name"]] = param
-                    method_doc["parameters"] = list(unique_params.values())
-                    method_doc["operationId"] = self.get_operation_id(path_item.get(method).get("summary", ""))
-                    path_item[method] = method_doc
-
-                    instance_schema = method_doc.get("responses", {}).get("200", {})
-                    if instance_schema and exposing_instance and method_doc["responses"]["200"].get("schema", None):
-                        method_doc["responses"]["200"]["schema"] = resource.SAFRSObject.swagger_models["instance"].reference()
-                        # add this later
-                        method_doc["responses"]["200"]["schema"] = {}
-
-                    try:
-                        validate_path_item_object(path_item)
-                    except FRSValidationError as exc:
-                        safrs.log.exception(exc)
-                        safrs.log.critical("Validation failed for {}".format(path_item))
-                        exit()
-
-                self._swagger_object["paths"][swagger_url] = path_item
-                # Check whether we manage to convert to json
-                try:
-                    json.dumps(self._swagger_object)
-                except Exception:
-                    safrs.log.critical("Json encoding failed for")
-                    # safrs.log.debug(self._swagger_object)
-
-        # disable API methods that were not set by the SAFRSObject
-        for http_method in HTTP_METHODS:
-            hm = http_method.lower()
-            if hm not in self.get_resource_methods(resource):
-                setattr(resource, hm, lambda x: ({}, HTTPStatus.METHOD_NOT_ALLOWED))
-
-        # pylint: disable=bad-super-call
-        super(FRSApiBase, self).add_resource(resource, *urls, **kwargs)
-
     @classmethod
-    def get_operation_id(cls, summary):
+    def _get_operation_id(cls, summary):
         """
-        get_operation_id
+        :param summary:
         """
         summary = "".join(c for c in summary if c.isalnum())
         if summary not in cls._operation_ids:
@@ -445,17 +513,17 @@ class SAFRSAPI(FRSApiBase):
 
 
 def api_decorator(cls, swagger_decorator):
-    """ Decorator for the API views:
-            - add swagger documentation ( swagger_decorator )
-            - add cors
-            - add generic exception handling
+    """Decorator for the API views:
+        - add swagger documentation ( swagger_decorator )
+        - add cors
+        - add generic exception handling
 
-        We couldn't use inheritance because the rest method decorator
-        references the cls.SAFRSObject which isn't known
+    We couldn't use inheritance because the rest method decorator
+    references the cls.SAFRSObject which isn't known
 
-        :param cls: The class that will be decorated (e.g. SAFRSRestAPI, SAFRSRestRelationshipAPI)
-        :param swagger_decorator: function that will generate the swagger
-        :return: decorated class
+    :param cls: The class that will be decorated (e.g. SAFRSRestAPI, SAFRSRestRelationshipAPI)
+    :param swagger_decorator: function that will generate the swagger
+    :return: decorated class
     """
 
     cors_domain = get_config("cors_domain")
@@ -487,16 +555,15 @@ def api_decorator(cls, swagger_decorator):
             decorated_method = cors.crossdomain(origin=cors_domain)(decorated_method)
         # Add exception handling
         decorated_method = http_method_decorator(decorated_method)
-
         setattr(decorated_method, "SAFRSObject", cls.SAFRSObject)
 
         try:
             # Add swagger documentation
             decorated_method = swagger_decorator(decorated_method)
-        except RecursionError:
+        except RecursionError:  # pragma: no cover
             # Got this error when exposing WP DB, TODO: investigate where it comes from
             safrs.log.error("Failed to generate documentation for {} {} (Recursion Error)".format(cls, decorated_method))
-        # pylint: disable=broad-except
+
         except Exception as exc:
             safrs.log.exception(exc)
             safrs.log.error("Failed to generate documentation for {}".format(decorated_method))
@@ -514,21 +581,21 @@ def api_decorator(cls, swagger_decorator):
 
 
 def http_method_decorator(fun):
-    """ Decorator for the supported jsonapi HTTP methods (get, post, patch, delete)
-        - commit the database
-        - convert all exceptions to a JSON serializable GenericError
+    """Decorator for the supported jsonapi HTTP methods (get, post, patch, delete)
+    - commit the database
+    - convert all exceptions to a JSON serializable GenericError
 
-        This method will be called for all requests
-        :param fun:
-        :return: wrapped fun
+    This method will be called for all requests
+    :param fun:
+    :return: wrapped fun
     """
 
     @wraps(fun)
     def method_wrapper(*args, **kwargs):
-        """ Wrap the method and perform error handling
-            :param *args:
-            :param **kwargs:
-            :return: result of the wrapped method
+        """Wrap the method and perform error handling
+        :param *args:
+        :param **kwargs:
+        :return: result of the wrapped method
         """
         safrs_exception = None
         try:
