@@ -8,7 +8,7 @@ import json
 import flask
 from http import HTTPStatus
 import yaml
-from sqlalchemy.orm.interfaces import ONETOMANY, MANYTOMANY  # , MANYTOONE
+from sqlalchemy.orm.interfaces import ONETOMANY, MANYTOMANY, MANYTOONE
 from flask_restful_swagger_2 import Schema, swagger
 from safrs.errors import ValidationError
 from safrs.config import get_config, is_debug
@@ -31,6 +31,7 @@ debug_responses = {
 }
 
 # List to generate the swagger references / definitions unique name
+# The _references will be added to the swagger in safrs
 Schema._reference_count = []
 Schema._references = {}
 
@@ -38,7 +39,7 @@ Schema._references = {}
 # pylint: disable=redefined-builtin,line-too-long,protected-access,logging-format-interpolation
 def parse_object_doc(object):
     """
-        Parse the yaml description from the documented methods
+    Parse the yaml description from the documented methods
     """
     api_doc = {}
     obj_doc = str(inspect.getdoc(object))
@@ -61,20 +62,20 @@ def parse_object_doc(object):
 
 def jsonapi_rpc(http_methods, valid_jsonapi=True):
     """
-        Decorator to expose functions in the REST API:
-        When a method is decorated with jsonapi_rpc, this means
-        it becomes available for use through HTTP POST (i.e. public)
+    Decorator to expose functions in the REST API:
+    When a method is decorated with jsonapi_rpc, this means
+    it becomes available for use through HTTP POST (i.e. public)
 
-        :param http_methods:
-        :return: function
+    :param http_methods:
+    :return: function
     """
 
     def _documented_api_method(method):
         """
-            :param method:
-            add metadata to the method:
-                REST_DOC: swagger documentation
-                HTTP_METHODS: the http methods (GET/POST/..) used to call this method
+        :param method:
+        add metadata to the method:
+            REST_DOC: swagger documentation
+            HTTP_METHODS: the http methods (GET/POST/..) used to call this method
         """
         USE_API_METHODS = get_config("USE_API_METHODS")
         if USE_API_METHODS:
@@ -92,8 +93,8 @@ def jsonapi_rpc(http_methods, valid_jsonapi=True):
 
 def is_public(method):
     """
-        :param method:
-        :return: True or False, whether the method is to be exposed
+    :param method:
+    :return: True or False, whether the method is to be exposed
     """
 
     return hasattr(method, REST_DOC)
@@ -101,8 +102,8 @@ def is_public(method):
 
 def get_doc(method):
     """
-        :param  method:
-        :return: OAS documentation
+    :param  method:
+    :return: OAS documentation
     """
 
     return getattr(method, REST_DOC, None)
@@ -110,8 +111,8 @@ def get_doc(method):
 
 def get_http_methods(method):
     """
-        :param method:
-        :return: a list of http methods used to call this method
+    :param method:
+    :return: a list of http methods used to call this method
     """
 
     return getattr(method, HTTP_METHODS, ["POST"])
@@ -119,10 +120,10 @@ def get_http_methods(method):
 
 def SchemaClassFactory(name, properties):
     """
-        Generate a Schema class, used to describe swagger schemas
-        :param name: schema class name
-        :param properties: class attributes
-        :return: class
+    Generate a Schema class, used to describe swagger schemas
+    :param name: schema class name
+    :param properties: class attributes
+    :return: class
     """
 
     # generate a unique name to be used as a reference
@@ -147,9 +148,9 @@ def SchemaClassFactory(name, properties):
 
 def encode_schema(obj):
     """
-        None aka "null" is invalid in swagger schema definition
-        This breaks our samples :/
-        We don't add the item to the schema if it's None
+    None aka "null" is invalid in swagger schema definition
+    This breaks our samples :/
+    We don't add the item to the schema if it's None
     """
     result = ""
 
@@ -166,9 +167,9 @@ def encode_schema(obj):
 # pylint: disable=redefined-builtin
 def schema_from_object(name, object):
     """
-        :param name:
-        :param object:
-        :return: swagger schema object
+    :param name:
+    :param object:
+    :return: swagger schema object
     """
 
     properties = {}
@@ -208,37 +209,37 @@ def schema_from_object(name, object):
 
 def get_swagger_doc_arguments(cls, method_name, http_method):
     """
-        :param cls: class containing the method to be exposed
-        :param method_name: name of the method to be exposed
-        :param http_method: HTTP method used to invoke
-        :return: parameters, fields, description, method
+    :param cls: class containing the method to be exposed
+    :param method_name: name of the method to be exposed
+    :param http_method: HTTP method used to invoke
+    :return: parameters, fields, description, method
 
-        create a schema for all methods which can be called through the
-        REST POST interface
+    create a schema for all methods which can be called through the
+    REST POST interface
 
-        A method is called with following JSON payload:
-        ```{
-            "meta"   : {
-                         "args" : {
-                                    "parameter1" : "value1" ,
-                                    "parameter2" : "value2" ,
-                                  }
-                       }
-        }```
+    A method is called with following JSON payload:
+    ```{
+        "meta"   : {
+                     "args" : {
+                                "parameter1" : "value1" ,
+                                "parameter2" : "value2" ,
+                              }
+                   }
+    }```
 
-        The schema is created using the values from the documented_api_method decorator,
-        returned by get_doc()
+    The schema is created using the values from the documented_api_method decorator,
+    returned by get_doc()
 
-        We use "meta" to remain compliant with the jsonapi schema
+    We use "meta" to remain compliant with the jsonapi schema
     """
 
     parameters = []
+    fields = {}
     # for method_name, method in inspect.getmembers(cls, predicate=inspect.ismethod):
     for name, method in inspect.getmembers(cls):
         if name != method_name:
             continue
 
-        fields = {}
         rest_doc = get_doc(method)
         description = rest_doc.get("description", "")
         if rest_doc:
@@ -247,7 +248,7 @@ def get_swagger_doc_arguments(cls, method_name, http_method):
             if method_args and isinstance(method_args, dict):
                 if http_method == "post":
                     """
-                        Post arguments, these require us to build a schema
+                    Post arguments, these require us to build a schema
                     """
                     model_name = "{}_{}".format(cls.__name__, method_name)
                     method_field = {"method": method_name, "args": method_args}
@@ -306,7 +307,7 @@ def swagger_doc(cls, tags=None):
 
     def swagger_doc_gen(func, instance=False):
         """
-            Decorator used to document SAFRSRestAPI HTTP methods exposed in the API
+        Decorator used to document SAFRSRestAPI HTTP methods exposed in the API
         """
         default_id = cls._s_sample_id()
         class_name = cls.__name__
@@ -345,13 +346,19 @@ def swagger_doc(cls, tags=None):
         sample_dict = cls._s_sample_dict()
 
         # Samples with "id" are used for GET and PATCH
-        coll_sample_data = schema_from_object(
-            coll_model_name, {"data": [{"attributes": sample_dict, "type": cls._s_type, "id": cls._s_sample_id()}]}
-        )
+        sample_instance = {"attributes": sample_dict, "type": cls._s_type, "id": cls._s_sample_id()}
 
-        inst_sample_data = schema_from_object(
-            inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type, "id": cls._s_sample_id()}}
-        )
+        if http_method == "get":
+            sample_rels = {}
+            for rel_name, val in cls._s_relationships.items():
+                sample_rels[rel_name] = {"data": None if val.direction is MANYTOONE else [], "links": {"self": None}}
+            sample_instance["relationships"] = sample_rels
+
+        coll_sample_data = schema_from_object(coll_model_name, {"data": [sample_instance]})
+        coll_sample_data.description = "Auto generated {}".format(coll_model_name)
+
+        inst_sample_data = schema_from_object(inst_model_name, {"data": sample_instance})
+        inst_sample_data.description = "Auto generated {}".format(inst_model_name)
 
         cls.swagger_models["instance"] = inst_sample_data
         cls.swagger_models["collection"] = coll_sample_data
@@ -360,11 +367,10 @@ def swagger_doc(cls, tags=None):
             doc["summary"] = "Retrieve a {} object".format(class_name)
             doc["collection_summary"] = "Retrieve a collection of {} objects".format(class_name)
             body, responses = cls._s_get_swagger_doc(http_method)
-            responses[HTTPStatus.OK.value] = {"schema": coll_sample_data}
+            responses[HTTPStatus.OK.value] = {"schema": coll_sample_data, "description": HTTPStatus.OK.description}
 
         elif http_method == "patch":
             post_model, responses = cls._s_get_swagger_doc(http_method)
-
             parameters.append(
                 {
                     "name": "PATCH body",
@@ -374,10 +380,11 @@ def swagger_doc(cls, tags=None):
                     "required": True,
                 }
             )
+            responses[HTTPStatus.OK.value] = {"schema": inst_sample_data, "description": HTTPStatus.OK.description}
+
         elif http_method == "post":
             _, responses = cls._s_get_swagger_doc(http_method)
             doc["summary"] = "Create a {} object".format(class_name)
-
             # Create the default POST body schema
             sample_dict = cls._s_sample_dict()
             # The POST sample doesn't contain an "id", unless cls.allow_client_generated_ids is True
@@ -395,6 +402,7 @@ def swagger_doc(cls, tags=None):
                     "required": True,
                 }
             )
+            responses[HTTPStatus.CREATED.value] = {"schema": inst_sample_data, "description": HTTPStatus.CREATED.description}
 
         elif http_method == "delete":
             _, responses = cls._s_get_swagger_doc(http_method)
@@ -424,7 +432,7 @@ def swagger_relationship_doc(cls, tags=None):
 
     def swagger_doc_gen(func):
         """
-            Decorator used to document relationship methods exposed in the API
+        Decorator used to document relationship methods exposed in the API
         """
 
         parent_class = cls.relationship.parent.class_
@@ -485,8 +493,13 @@ def swagger_relationship_doc(cls, tags=None):
             data = {"type": child_class._s_type, "id": child_sample_id}
 
             if cls.relationship.direction in (ONETOMANY, MANYTOMANY):
+                # tomany relationships only return a 204 accepted
                 data = [data]
+                responses.pop(HTTPStatus.OK.value, None)
+
             rel_post_schema = schema_from_object("{}_Relationship".format(class_name), {"data": data})
+            cls.swagger_models["instance"] = rel_post_schema
+            cls.swagger_models["collection"] = rel_post_schema
             parameters.append(
                 {
                     "name": "{} body".format(class_name),
@@ -497,6 +510,10 @@ def swagger_relationship_doc(cls, tags=None):
                 }
             )
 
+            if cls.relationship.direction is MANYTOONE:
+                # toone relationships return 200 OK
+                responses[HTTPStatus.OK.value] = {"schema": rel_post_schema, "description": HTTPStatus.OK.description}
+
         elif http_method == "delete":
             child_sample_id = child_class._s_sample_id()
 
@@ -540,12 +557,12 @@ def swagger_relationship_doc(cls, tags=None):
 
 def swagger_method_doc(cls, method_name, tags=None):
     """
-        Generate decorator used to document SAFRSJSONRPCAPI instances
+    Generate decorator used to document SAFRSJSONRPCAPI instances
     """
 
     def swagger_doc_gen(func):
         """
-            decorator
+        decorator
         """
         method = getattr(cls, method_name, None)
         method_doc = parse_object_doc(method)
@@ -603,7 +620,7 @@ def swagger_method_doc(cls, method_name, tags=None):
 
 def default_paging_parameters():
     """
-        default_paging_parameters
+    default_paging_parameters
     """
 
     parameters = []
@@ -633,7 +650,7 @@ def default_paging_parameters():
 
 def apply_fstring(swagger_obj, vars, k=None):
     """
-        Format the f-strings in the swagger object
+    Format the f-strings in the swagger object
     """
     if isinstance(swagger_obj, str):
         result = swagger_obj
